'--------------------------------------------------------------------------------------------
' Module:  PROPCON.Bas
' Desc:    Properties List control
' Interface:
'          The control is driven by WM_USER messages, WM_USER is offset as follows:
'          %our_add_property        = 100
'          %our_get_property        = 102
'          %our_get_property_count  = 103
'          This being a properties list, the content is unlikely to be changed dynamically
'          so I have saved 30 minutes by not coding a "del_property" or "set_property"
'          method.
'          Loading the property sheet with properties and their default values is
'          shown in InitializePropertyItems.
'          Extracting the properties and their values is done in the DEBUG button handler.
'          The developer also has to handle the WM_MEASUREITEM message as shown.
' Issues:
'          1. lack of thorough testing
'          2. WM_MEASUREITEM
' Purpose: a standalone control for managing a properties list
' History: Listbox handling code borrowed from Jules Marchildon's post of 2001 , not changed much
' Date:    7-FEB-2008
' Name:    Chris Holbrook
'
'--------------------------------------------------------------------------------------------

#Compile Exe
#Include "WIN32API.INC"
#Include "COMDLG32.INC"


'---

Type PROPERTYITEM
  propName  As Asciiz*225
  curValue  As Asciiz*255
  nItemType As Integer
  cmbItems  As Asciiz*255
  fptr      As propertyitem Ptr
End Type

' message types - WM_USER offsets
%our_add_property        = 100
%our_get_property        = 101
%our_get_property_count  = 102
%our_MEASUREITEM         = 103
%our_drawitem            = 104
%our_lb_selchange        = 105
%our_msg_max             = 106  ' must be highest WM_USER offset + 1

'---
%IDC_LISTBOX = 1000
%IDC_COMBO   = 1001
%IDC_BUTTON  = 1002
%IDC_EDIT    = 1003
%IDC_UCC     = 1004
%IDC_DEBUG_BN = 1005

'PIT = property item type, Button is default
%PIT_COMBO   = 0
%PIT_EDIT    = 1
%PIT_COLOR   = 2
%PIT_FONT    = 3
%PIT_FILE    = 4
'---
Declare Sub InitializePropertyItems( hOwner As Dword, lCtlId As Long)

Declare Sub InitializeCustomColors( pColors As Dword)
Declare Sub OnSelChange(hlist As Dword, hcombo As Dword, hbutton As Dword, hedit As Dword, _
                nDivider As Integer)
Declare Sub OnButtonClicked(hlist As Dword, pColors As Dword)
Declare Sub InvertLine( ByVal hList As Dword, ByVal xleft1 As Integer ,ByVal yFrom As Integer, _
                ByVal xleft2 As Integer ,ByVal yTo As Integer )
Declare Function MakeFont(ByVal sFont As String, ByVal PointSize As Long) As Long

'-------------------------------------------------------------
' common functions
'------------------------------------------------------------------------------
' borrowed from Semen Matusovski
Function CreateSuperClass(OldClassName As String, NewClassName As String, lpfnNewWndProc As Long, cbWndExtra As Long) As Long
Local wc As WNDCLASSEX
Local result As Long
    wc.cbSize = SizeOf(wc)
    If GetClassInfoEx(ByVal 0&, ByVal StrPtr(OldClassName), wc) Then
        CallWindowProc lpfnNewWndProc, 0, 0, wc.lpfnWndProc, wc.cbWndExtra
        wc.hInstance = GetModuleHandle(ByVal 0&)
        wc.lpszClassName = StrPtr(NewClassName)
        wc.lpfnWndProc = lpfnNewWndProc
        wc.cbWndExtra = wc.cbWndExtra + cbWndExtra
        result = RegisterClassEx(wc)
        Function = result
    End If
End Function

'-------------------------------------------------------------
' WinMain:
'
'
'-------------------------------------------------------------
Function WinMain (ByVal hInstance     As Long, _
                  ByVal hPrevInstance As Long, _
                  ByVal lpCmdLine           As Asciiz Ptr, _
                  ByVal iCmdShow      As Long) As Long

    Local Msg       As tagMsg
    Local wndclss  As WndClassEx
    Local hWnd, result      As Long
    Local rct       As RECT
    Local szpgmname As Asciiz * 20

    szpgmname = "PROPLIST"

    wndclss.cbSize        = SizeOf(WndClss)
    wndclss.style         = %CS_HREDRAW Or %CS_VREDRAW Or %CS_DBLCLKS
    wndclss.lpfnWndProc   = CodePtr( LBWndProc )
    wndclss.cbClsExtra    = 0
    wndclss.cbWndExtra    = 0
    wndclss.hInstance     = hInstance
    wndclss.hIcon         = LoadIcon( hInstance, "MAINICON" )
    wndclss.hCursor       = LoadCursor( %NULL, ByVal %IDC_ARROW )
    wndclss.hbrBackground = GetStockObject( %LTGRAY_BRUSH )
    wndclss.lpszMenuName  = %NULL
    wndclss.lpszClassName = VarPtr(szpgmname)
    wndclss.hIconSm       = LoadIcon(hInstance, ByVal %IDI_APPLICATION)

    result = RegisterClassEx (wndclss)

    If result = 0 Then
        result = getlasterror
        Err = result
        ?Error$
    End If
    If IsFalse(CreateSuperClass("LISTBOX","OurLB", CodePtr(ListBoxProc), 4)) Then Exit Function

    style& = %WS_OVERLAPPED Or _
             %WS_CAPTION Or _
             %WS_SYSMENU Or _ '%WS_SIZEBOX OR _
             %WS_visible 'MINIMIZEBOX

    'create the main window (my screen LCD,1024x768)
    hWnd = CreateWindowEx(0, _
                szpgmname, _
                "Control Properties:", _
                style&, _
                210, 110, 650, 350, _
                %HWND_DESKTOP, _
                %NULL, _
                hInstance, _
                ByVal %NULL)

    ' fail if window is not created
    If IsFalse hWnd Then
      Function = %FALSE
      Exit Function
    End If


    ' Activate window
    ShowWindow hWnd, %SW_SHOW
    ' Paint client area
    UpdateWindow hWnd
    While IsTrue GetMessage(msg, ByVal %NULL, 0, 0)
        If IsFalse IsDialogMessage(hWnd, msg) Then
            TranslateMessage msg
            DispatchMessage msg
        End If
    Wend
    Function = msg.wParam

End Function
'------------------------------------------------------------------------
'
' MainWndProc()
'
'------------------------------------------------------------------------
Function LBWndProc (ByVal hWnd As Long, ByVal wMsg As Long, _
                     ByVal wParam As Long, ByVal lParam As Long) As Long

    Static hlist, hfont, hinstance, hwndBn         As Dword
    Local lpMs            As MEASUREITEMSTRUCT Ptr
    Local psz As Asciz Ptr
    Local result, nrows, i As Long
    Local sz As Asciz * 255
    Local pitem          As PROPERTYITEM Ptr
    Local s As String

'---

    Select Case wMsg
        Case %WM_CREATE
            hinstance = getmodulehandle(ByVal 0)
            '  Create the OwnerDraw List box for our control properties...
            hList  = CreateWindowEx(%WS_EX_CLIENTEDGE, "OurLB", "", _
                     %WS_CHILD Or %WS_VISIBLE Or %WS_TABSTOP Or %WS_BORDER Or _
                     %LBS_NOINTEGRALHEIGHT Or %LBS_HASSTRINGS Or _
                     %LBS_OWNERDRAWFIXED Or %LBS_NOTIFY Or %WS_VSCROLL, _
                     10,10,300,306, _
                     hWnd, %IDC_LISTBOX, hInstance, ByVal %NULL)

            hFont = MakeFont("Courier New", 9)
            Call SendMessage( hList, %WM_SETFONT, hFont, 0 )
            Call InitializePropertyItems(hWnd, %IDC_LISTBOX)

            ' Create a label control
            hWndBN = CreateWindowEx(%NULL, "Static","DEBUG", _
                     %SS_LEFT Or %WS_VISIBLE Or %WS_CHILD Or %SS_NOTIFY, _
                     335, 280, 50, 15, _
                     hWnd, %IDC_DEBUG_BN, hInstance, ByVal %NULL)


            Function = %TRUE
            Exit Function

        Case %WM_MEASUREITEM
            lpMs = LPARAM
            'arbitrary number for height
            @lpMs.itemHeight = 20

            Function = %TRUE
            Exit Function

        Case %WM_DRAWITEM
            sendmessage (hlist, %WM_USER + %our_DRAWITEM, wparam, lparam) ' pass the message on to the superclassed LB control

        Case %WM_COMMAND
            'ListBox control notifications...
            Select Case LoWrd(wParam)
                   Case %IDC_LISTBOX
                        Select Case HiWrd(wParam)
                               Case %LBN_SELCHANGE
                                   ' nudge the listbox control
                                   sendmessage(hlist, %WM_USER + %our_lb_selchange, 0, 0)
                        End Select
                   Case %IDC_DEBUG_BN
                        result = sendmessage(hlist, %WM_USER + %our_get_property_count,0,0)
                        ? "there are" + Str$(result) + " properties in the list"
                        ' work through listbox getting each property in turn,
                        ' requesting its values & formatting them for display
                        nrows = sendmessage(hlist, %LB_GETCOUNT,0,0)
                        For i = 1 To nrows
                            Call sendmessage(hlist, %LB_GETTEXT, i, VarPtr(sz))
                            pitem = sendmessage ( hlist, %WM_USER + %our_get_property, VarPtr(sz), 0)
                            s = s + "name = " + Trim$(@pitem.Propname) + "," + _
                                    "CurValue = " + Trim$(@pitem.CurValue) + ","
                            Select Case @pitem.nitemtype
                                Case %PIT_COMBO
                                    s = s + "nItemType = COMBO"
                                Case %PIT_EDIT
                                    s = s + "nItemType = EDIT"
                                Case %PIT_COLOR
                                    s = s + "nItemType = COLOR"
                                Case %PIT_FONT
                                    s = s + "nItemType = FONT"
                                Case %PIT_FILE
                                    s = s + "nItemType = FILE"
                            End Select
                            s = s + ",cmbitems = " + Trim$(@pitem.CmbItems) + "," + _
                                    "fptr = " + Str$( @pitem.fptr) + $CrLf
                        Next
                        ? s
            End Select
        Case %WM_DESTROY
            deleteObject hfont
    End Select
    Function = DefWindowProc(hWnd, wMsg, wParam, lParam)
End Function


' LBN_SELCHANGE:
'----------------------------------------------------------------------------
Sub OnSelChange(hlist As Dword, hcombo As Dword, hbutton As Dword, hedit As Dword, _
                nDivider As Integer)

    Local lBoxSelText As String
    Local rc As RECT
    Local cursel As Long
    Local nlastbox As Long
    Local pitem As PROPERTYITEM Ptr
    Local sz As Asciz * 255

    curSel = SendMessage( hList, %LB_GETCURSEL, 0, 0 )
    sendmessage ( hlist, %LB_GETTEXT, cursel, VarPtr(sz)) ' get selected text = propertyname
    lBoxSelText = sz
    pitem = sendmessage ( hList, %WM_USER + %our_get_property, VarPtr(sz), 0 ) ' get pointer to item structure
    Call SendMessage( hList, %LB_GETITEMRECT, curSel, VarPtr(rc) )

    rc.nleft = nDivider

    If hCombo  <> 0 Then Call ShowWindow( hCombo,  %SW_HIDE )
    If hButton <> 0 Then Call ShowWindow( hButton, %SW_HIDE )
    If hEdit   <> 0 Then Call ShowWindow( hEdit,   %SW_HIDE )

    If @pItem.nItemType = %PIT_COMBO  Then

        'display the combo box and move it to the new location
        nLastBox = 0

        If hCombo <> 0 Then
           Call MoveWindow( hCombo,rc.nleft-1,rc.ntop-2,rc.nright+2-rc.nleft,rc.nbottom+100,1 )
        End If

        'add the choices for this particular property
        cmbItems$   = @pItem.cmbItems
        lBoxSelText = @pItem.curValue

        Call SendMessage( hCombo,%CB_RESETCONTENT, 0,0 )

        delimiter$ = "|"
        nCount& = Tally(cmbItems$,delimiter$)

        For i& = 1 To nCount&
            dataItem$ = Parse$(cmbItems$,delimiter$,i&)
            'add each string to the ComboBox
            Call SendMessage(hCombo, %CB_ADDSTRING, i&-1, StrPtr(dataItem$) )
        Next

        Call ShowWindow( hCombo,%SW_SHOW )
        Call SetFocus( hCombo )
        'Call SetFocus( ghList )

        'jump to the property's current value in the combo box
        j& = SendMessage(hCombo,%CB_FINDSTRINGEXACT,0,StrPtr(lBoxSelText) )
        If  j& <> %CB_ERR  Then
            Call SendMessage( hCombo,%CB_SETCURSEL,j&,0 )
        Else
            'there is no current value, so default to first in list
            Call SendMessage( hCombo,%CB_SETCURSEL,0,0 )
        End If

    ElseIf @pItem.nItemType = %PIT_EDIT  Then

        'display edit box
        nLastBox   = 1
        rc.nbottom = rc.nbottom - 3

        If hEdit <> 0 Then
           Call MoveWindow( hEdit,rc.nleft+1,rc.ntop+3,rc.nright-rc.nleft,rc.nbottom-rc.ntop,1 )
        End If

        lBoxSelText = @pItem.curValue

        Call ShowWindow( hEdit, %SW_SHOW )
        Call SetFocus( hEdit )
        'Call SetFocus( ghList )

        'set the text in the edit box to the property's current value
        Call SendMessage(hEdit, %WM_SETTEXT, 0, StrPtr(lBoxSelText) )

    Else
        'displays a button if the property is a Color/File/Font chooser
        nLastBox = 2

        nWidth& = rc.nright - rc.nleft

        If  nWidth& > 25  Then
            rc.nleft   = rc.nright  - 25
        End If

        rc.nbottom = rc.nbottom - 3

        If hButton <> 0  Then
           Call MoveWindow( hButton,rc.nleft,rc.ntop,rc.nRight -rc.nleft, rc.nBottom -rc.ntop, 1 )
        End If

        Call ShowWindow( hButton, %SW_SHOW )
        Call SetFocus( hButton )
        'Call SetFocus( ghList )
        '---

    End If

End Sub
'----------------------------------------------------------------------------
' BN_CLICKED:
'----------------------------------------------------------------------------
Sub OnButtonClicked(hlist As Dword, pColors As Dword)

     Local lBoxSelText    As String
     Local initClr        As Dword
     Local currClr        As Asciiz*255
     Local clrStr         As Asciiz*255
     Local ColorSpec      As CHOOSECOLORAPI
     Local lResult, cursel        As Long
     Local lCounter       As Long
     Local lCustomColor() As Long
     Local pitem As PROPERTYITEM Ptr
     Local sz As Asciz * 255

    'display the appropriate common dialog depending on what type
    'of chooser is associated with the property
    ' get current listbox selection
    cursel = sendmessage ( hlist, %LB_GETCURSEL,0,0)
    sendmessage ( hlist, %LB_GETTEXT, cursel, VarPtr(sz)) ' get selected text = propertyname
    lBoxSelText = Trim$(sz)
    pitem = sendmessage ( hList, %WM_USER + %our_get_property, VarPtr(sz), 0 ) ' get pointer to item structure
    'First check for the Choose Color dialog...
    Select Case @pItem.nItemType
        Case %PIT_COLOR

            currClr = @pItem.curValue

            If  currClr > "" Then

                'parse the property's current color value
                currClr = LTrim$(currClr,"RGB(")
                currClr = RTrim$(currClr,")")

                initClr = RGB(Val(Parse$(currClr,d$,1)), _
                             Val(Parse$(currClr,d$,2)),  _
                             Val(Parse$(currClr,d$,3)))
            Else
                'use a default instead
                initClr = RGB( 255,128,128 )
            End If

            ColorSpec.lStructSize  = Len(ColorSpec)
            ColorSpec.hwndOwner    = hList
            ColorSpec.lpCustColors = pColors
            ColorSpec.rgbResult    = initClr
            ColorSpec.Flags        = ColorSpec.Flags Or %CC_RGBINIT

            lResult = ChooseColor(ColorSpec)

            If lResult = 0 Then 'check if user cancelled dialog ?
               Exit Sub
            Else

                'selClr = ColorSpec.rgbResult
                clrStr = Hex$(ColorSpec.rgbResult,6)

                'Note order of RGB, COLORREF 0x00bbggrr
                clrStr = "RGB(" + Trim$(Str$(Val("&H" + Right$(ClrStr,2)))) + _
                            "," + Trim$(Str$(Val("&H" + Mid$  (ClrStr,3,2)))) + _
                            "," + Trim$(Str$(Val("&H" + Left$ (ClrStr,2)))) + ")"
                @pItem.curValue = clrStr

                'Call ShowWindow( ghButton,%SW_HIDE )

                Call InvalidateRect( hList,ByVal %NULL,1)
                Call UpdateWindow ( hList )
             End If

        Case %PIT_FILE
            'Next check for the Open File Dialog...

            Local SelectedFile As Asciiz*255
            Local zTxt  As Asciiz * 255

            zTxt = "All pictures  (*.bmp,*.ico)|*.BMP;*.ICO|"
            zTxt = zTxt & "Bitmap (*.bmp)|*.BMP|"
            ZTxt = zTxt & "Icon   (*.ico)|*.ICO|"

            currPath$ = @pItem.curValue
            If currPath$ = "none" Then
               fName$ = ""
            Else
               fName$ = currPath$
            End If

            'use simple Open dialog for demo...
            tmp& = OpenFileDialog(getparent(hlist),"Select File:",fName$,CurDir$,zTxt,"BMP", _
                                  %OFN_FILEMUSTEXIST Or %OFN_HIDEREADONLY Or %OFN_LONGNAMES)
            If tmp& Then
                @pItem.curValue = fName$
                Call InvalidateRect(hList,ByVal %NULL,1)
                Call UpdateWindow ( hList )
            End If

    '    END IF '<remove later>

        'Last, check for the ChooseFont() dialog...
        Case %PIT_FONT

            Dim cf As CHOOSEFONTAPI
            Dim lfFont As LOGFONT
            cf.lStructSize = SizeOf(cf)
            cf.hWndOwner   = getparent(hlist)
            cf.lpLogFont   = VarPtr(lfFont)
            cf.Flags       = %CF_EFFECTS Or %CF_SCREENFONTS

            lResult = ChooseFont(cf)

            If lResult = 0 Then 'check if user cancelled dialog ?
               Exit Sub
            Else
                faceName$   = lfFont.lfFaceName
                'ToDo: Not included in orgional C++, get the Font height...
                @pItem.curValue = faceName$

                Call InvalidateRect( hList,ByVal %NULL,1)
                Call UpdateWindow ( hList )

            End If

    End Select
End Sub
'-------------------------------------------------------------------------
' Create a Font:
'-------------------------------------------------------------------------
Function MakeFont(ByVal sFont As String, ByVal PointSize As Long) As Long

    Local hDC As Long, CyPixels As Long
    hDC = GetDC(%HWND_DESKTOP)
    CyPixels  = GetDeviceCaps(hDC, %LOGPIXELSY)
    ReleaseDC %HWND_DESKTOP, hDC
    PointSize = (PointSize * CyPixels) \ 72

    Function = CreateFont(0 - PointSize, 0, 0, 0, %FW_NORMAL, 0, 0, 0, _
    %ANSI_CHARSET, %OUT_TT_PRECIS, %CLIP_DEFAULT_PRECIS, _
    %DEFAULT_QUALITY, %FF_DONTCARE, ByCopy sFont)

End Function


'-------------------------------------------------------------------------
'' SuperClassed ListBox Control Procedure:
'-------------------------------------------------------------------------
CallBack Function ListBoxProc

    Local lBoxSelText As String
    Static OldProc As Long, OffsetWndExtra As Long
    Local sz As Asciz * 64
    Local psz As Asciz Ptr
    Local cursel, hDC, nindex As Long
    Local r, rc, rc2, rectfull As rect
    Local pt As POINTAPI
    Static hbrush, hCursorSize, hcursorarrow, hcombo, hedit, hbutton, hfont, hinst As Dword
    Local p_item, q_item, r_item As PROPERTYITEM Ptr
    Local h, i            As Long
    Static pitem, ppty    As propertyitem Ptr

    Static nDivider       As Integer
    Static nDivTop        As Integer
    Static nDivBtm        As Integer
    Static nOldDivX       As Integer
    Static nLastBox       As Integer
    Static bTracking      As Long
    Static bDivIsSet      As Integer
    Dim Ccolor(0 To 15)   As Static Long
    Local lpDs            As DRAWITEMSTRUCT Ptr
    Local pdw             As Dword Ptr
    Local lcreating As Long

    '
    ' message from  CreateSuperClass to give us the WNDPROC for the base class
    If CbHndl = 0 Then
        OldProc = CbWParam
        OffsetWndExtra = CbLParam
        Exit Function
    End If
    '
    Select Case CbMsg

        Case %WM_CREATE
            lcreating = %true
            hCursorArrow =  LoadCursor( %NULL, ByVal %IDC_ARROW )
            hinst = getmodulehandle(ByVal 0)
            hCursorSize  =  LoadCursor( %NULL, ByVal %IDC_SIZEWE )
            bTracking = %FALSE
            nDivider  = 0
            bDivIsSet = %FALSE
            '  Create the controls that will need to edit the Properties with.
            '  They are all hidden for now and resized later.
            hButton = CreateWindowEx(0,"BUTTON","...", _
                                      %WS_CHILD Or %WS_CLIPSIBLINGS Or %BS_PUSHBUTTON, _
                                      0,0,8,12, _
                                      CbHndl, %IDC_BUTTON, hInst, ByVal %NULL)

            hCombo  = CreateWindowEx(0,"COMBOBOX", "", _
                                      %WS_CHILD Or %CBS_DROPDOWNLIST Or %CBS_NOINTEGRALHEIGHT, _
                                      0,0,10,100, _
                                      CbHndl, %IDC_COMBO, hInst, ByVal %NULL)

            hEdit   = CreateWindowEx(0,"EDIT", "True", _
                                      %WS_CHILD Or %WS_CLIPSIBLINGS Or %ES_LEFT Or %ES_AUTOHSCROLL Or %WS_BORDER, _
                                      0,0,10,20, _
                                      CbHndl, %IDC_EDIT, hInst, ByVal %NULL)
            Call InitializeCustomColors( VarPtr(Ccolor(0)))
            lcreating = %false
        Case %WM_SETFONT
            hfont = CbWParam
            Call SendMessage( hEdit, %WM_SETFONT, hFont, 0 )
            Call SendMessage( hCombo, %WM_SETFONT, hFont, 0 )
            ' NB SetFont does not return a value

        'Catch the Combo,Edit,Button child control notifications here...

        Case %WM_COMMAND
            cursel = sendmessage ( CbHndl, %LB_GETCURSEL,0,0)
            sendmessage ( CbHndl, %LB_GETTEXT, cursel, VarPtr(sz)) ' get selected text = propertyname
            lBoxSelText = Trim$(sz)
            pitem = sendmessage ( CbHndl, %WM_USER + %our_get_property, VarPtr(sz), 0 ) ' get pointer to item structure

            Select Case CbCtl
                   Case %IDC_COMBO
                        If (CbCtlMsg = %CBN_SELCHANGE) Then
                            Local selStr As Asciiz*255
                            If hCombo <> 0  Then
                               idx& = SendMessage( hCombo, %CB_GETCURSEL ,0 ,0 )
                               Call SendMessage( hCombo, %CB_GETLBTEXT, idx& , VarPtr(selStr) )
                               @pItem.curValue = selStr
                            End If
                        End If

                   Case %IDC_EDIT
                        If (CbCtlMsg = %EN_CHANGE) Then
                            Local newStr As Asciiz*255
                            tLen&  = SendMessage(hEdit, %WM_GETTEXTLENGTH, 0, 0) + 1
                            newStr = Space$(tLen&)
                            Call SendMessage(hEdit, %WM_GETTEXT, tLen&, VarPtr(newStr))
                            If pitem <> 0 Then ' no items exist during initialisation!
                                @pItem.curValue = newStr
                            End If
                        End If

                   Case %IDC_BUTTON
                        If (CbCtlMsg = %BN_CLICKED) Then
                            cursel = sendmessage ( CbHndl, %LB_GETCURSEL,0,0)
                            Call OnButtonClicked( CbHndl, VarPtr(Ccolor(0)))
                        End If

            End Select

        Case %WM_LBUTTONDOWN
            If hCombo  <> 0 Then Call ShowWindow( hCombo,  %SW_HIDE )
            If hEdit   <> 0 Then Call ShowWindow( hEdit,   %SW_HIDE )
            If hButton <> 0 Then Call ShowWindow( hButton, %SW_HIDE )

            '-----------------------[ Splitter ]-----------------------
            pt.x = LoWrd(CbLParam)
            pt.y = HiWrd(CbLParam)

            If ((pt.x >= nDivider-5) And (pt.x <= nDivider+5)) Then
                'if mouse clicked on divider line, then start resizing
                Call SetCursor(hCursorSize)

                Call GetWindowRect( CbHndl ,rc )
                rc.nleft  = rc.nleft  +10
                rc.nright = rc.nright -20

                'do not let mouse leave the list box boundary
                Call ClipCursor( rc )
                Call GetClientRect( CbHndl ,rc )
                bTracking = %TRUE

                nDivTop   = rc.ntop
                nDivBtm   = rc.nbottom
                nOldDivX  = pt.x

                Call InvertLine( CbHndl, nOldDivX ,nDivTop ,nOldDivX ,nDivBtm )

                'capture the mouse
                Call SetCapture(CbHndl)

            End If


        Case %WM_MOUSEMOVE
            pt.x = LoWrd(CbLParam)
            pt.y = HiWrd(CbLParam)

            'move divider line to the mouse position
            'if columns are currently being resized
            If bTracking = %TRUE  Then

                'remove old divider line
                Call InvertLine( CbHndl, nOldDivX ,nDivTop ,nOldDivX ,nDivBtm )

                'draw new divider line
                Call InvertLine( CbHndl, pt.x ,nDivTop ,pt.x ,nDivBtm )

                nOldDivX = pt.x

            Else

                'set the cursor to a sizing cursor if
                'the cursor is over the row divider
                If (pt.x >= nDivider-5) And (pt.x <= nDivider+5) Then
                   Call SetCursor(hCursorSize)
                End If

           End If


        Case %WM_LBUTTONUP
            pt.x = LoWrd(CbLParam)
            pt.y = HiWrd(CbLParam)

            'if columns were being resized then this indicates
            'that mouse is up so resizing is done.  Need to redraw
            'columns to reflect their new widths.
            If bTracking = %TRUE Then

                bTracking = %FALSE
                Call ReleaseCapture()
                Call ClipCursor( ByVal %NULL )

                Call InvertLine( CbHndl, pt.x ,nDivTop ,pt.x ,nDivBtm )


                'set the divider position to the new value
                nDivider = pt.x

                'Self paint the ListBox control...
                Call InvalidateRect(CbHndl ,ByVal %NULL, 1)
                Call UpdateWindow(CbHndl)

            End If

           Case %WM_CAPTURECHANGED
                If CbLParam <> getparent(CbHndl) Then
                    If bTracking = %TRUE Then
                        bTracking = %FALSE
                        Call ReleaseCapture()
                        Call ClipCursor( ByVal %NULL )
                        Call InvalidateRect(CbHndl ,ByVal %NULL, 1)
                        Call UpdateWindow(CbHndl)
                    End If
                End If
           '---------------------[ end of splitter ]------------------
            '---

        ' user messages
        Case %WM_USER + %our_lb_selchange ' sent when LB_SELCHANGE rec'd by parent
            curSel = SendMessage( CbHndl, %LB_GETCURSEL, 0, 0 )
            Call OnSelChange(CbHndl, hcombo, hbutton, hedit, ndivider)

        Case %WM_USER + %our_DRAWITEM ' parent has rec'd a WM_DRAWITEM msg
            lpDs = CbLParam
            If @lpDs.itemID < 0 Then
                Exit Select
            End If
            If bTracking = %TRUE Then Exit Function
            rectFull = @lpDs.rcItem
            rc = rectFull
            If nDivider = 0 Then nDivider   = (rc.nRight - rc.nLeft)  \ 2
            rc.nleft   = nDivider
            rc2        = rectFull
            rc2.nright = rc.nleft - 1
            nIndex     = @lpDs.itemID
            sendmessage ( CbHndl, %LB_GETTEXT, nIndex, VarPtr(sz)) ' get selected text = propertyname
            lBoxSelText = Trim$(sz)
            pitem = sendmessage ( CbHndl, %WM_USER + %our_get_property, VarPtr(sz), 0 ) ' get pointer to item structure
            If  nIndex  > -1  Then
                'draw two rectangles, one for each row column
                hBrush = CreateSolidBrush( RGB(192,192,192) )
                Call FillRect(@lpDs.hDC , rc2, hBrush )
                Call DeleteObject(hBrush)

                Call DrawEdge(@lpDs.hDC , rc2, %EDGE_SUNKEN,%BF_BOTTOMRIGHT )
                Call DrawEdge(@lpDs.hDC , rc,  %EDGE_SUNKEN,%BF_BOTTOM )

                Call SetBkMode( @lpDs.hDC , %TRANSPARENT )

                rc2.nleft   = rc2.nleft   + 3
                rc2.ntop    = rc2.ntop    + 3
                rc2.nright  = rc2.nright  - 3
                rc2.nbottom = rc2.nbottom + 3

                'write the Property Name in the first rectangle, left side
                Call DrawText( @lpDs.hDC , @pItem.propName, _
                               Len(@pItem.propName), _
                               rc2, %DT_LEFT Or %DT_SINGLELINE )

                rc.nleft   = rc.nleft   + 3
                rc.ntop    = rc.ntop    + 3
                rc.nright  = rc.nright  + 3
                rc.nbottom = rc.nbottom + 3

                'write the initial property value in the second rectangle, right side
                Call DrawText(@lpDs.hDC , @pItem.curValue, _
                              Len(@pItem.curValue), _
                              rc, %DT_LEFT Or %DT_SINGLELINE )
            End If

            '<insert here>  Add some kind of item focus rect

            Function = %TRUE
            Exit Function

        Case %WM_USER + %our_add_property
            Beep
            pdw = CbLParam
            p_item = CbLParam
            ' grab memory to hold new property and ititialise to zeros
            h = GlobalAlloc ( %GPTR, SizeOf(PropertyItem))
            ' move property details into memory
            movememory ( ByVal h, ByVal pdw, SizeOf(PROPERTYITEM))
            ' add it into list at end
            If ppty = %NULL Then ' empty list
                ppty = h
            Else
                p_item = ppty
                While @p_item.fptr <> %NULL
                    p_item = @p_item.fptr
                Wend
                @p_item.fptr = h
            End If
            q_item = h
            sz = @q_item.propname
            pitem = q_item          ' pitem is static!
            Call SendMessage(CbHndl,%LB_ADDSTRING,0,VarPtr(sz))
            Exit Function

        Case %WM_USER + %our_get_property ' wparam has a ptr to propertyname
            ' find property by name
            p_item = ppty
            If p_item = %NULL Then Exit Function ' empty list
            psz = CbWParam
            While Trim$(@p_item.propname)  <> Trim$(@psz)
                p_item = @p_item.fptr
                If p_item = %NULL Then Exit Function ' return null pointer
            Wend
            ' return msg with pointer to it
            Function = p_item ' return pointer to property
            Exit Function
            'sendmessage ( hParent, %WM_USER + %our_ispty, 0, p_item)


        Case %WM_USER + %our_get_property_count
            ' count properties in list
            p_item = ppty
            If p_item = %NULL Then Exit Function ' return zero
            While %TRUE
                Incr i
                p_item = @p_item.fptr
                If p_item = %NULL Then
                    Function = i
                    Exit Function
                End If
            Wend
        '
        Case %WM_DESTROY
            ' deallocate memory in property linked list
            p_item = ppty
            While p_item <> %null
                q_item = p_item
                p_item = @p_item.fptr
                globalfree( ByVal q_item )
            Wend
            ppty = %NULL
            '
            Call PostQuitMessage( 0 )
    End Select

    ' if the message was received as a user message, it is not processed further
    If (CbMsg > %WM_USER) And ( CbMsg < %WM_USER + %our_msg_max) Then Exit Function
    '
    Function = CallWindowProc(OldProc, CbHndl, CbMsg, CbWParam, CbLParam)
End Function


'--------------------------------------------------------------------------
' InvertLine()
'--------------------------------------------------------------------------
Sub InvertLine( ByVal hList As Dword, ByVal xleft1 As Integer ,ByVal yFrom As Integer, _
                ByVal xleft2 As Integer ,ByVal yTo As Integer )


    Local hDC       As Long
    Local nDrawMode As Long
    Local oldpt     As POINTAPI

    '*Get DC of ListBox
    hDC = GetDC(hList)

    '*Use the default Pen color and style...

    '*Set the GDI foreground mix mode
    nDrawMode = SetROP2(hDC, %R2_NOT)

    '*Draw the Line
    Call MoveToEx( hDC ,xLeft1 ,yFrom ,oldpt)
    Call LineTo( hDC ,xLeft2 ,yTo)

    '*Restore DC
    Call SetROP2(hDC, nDrawMode)
    Call ReleaseDC(hList, hDC)

End Sub

'--------------------------------------------------------------------------
' User Designed Custom Control Properties:
'
'--------------------------------------------------------------------------
' This sub is outside the custom control so it needs to know where the CC is
' for which purpose the parent hWnd and control id are passed in
Sub InitializePropertyItems( hOwner As Dword, lCtlId As Long)
    Local pi As propertyitem


    pi.propName  =  "ToolTip Text"
    pi.curValue  =  "Litte Red Riding Hood..."
    pi.nItemType =  %PIT_EDIT
    pi.cmbItems  =  ""
    Call SendMessage(getdlgitem(howner, lCtlId),%WM_USER + %our_add_property, 0,VarPtr(pi))

    pi.propName  =  "Enabled"
    pi.curValue  =  "true"
    pi.nItemType =  %PIT_COMBO
    pi.cmbItems  =  "true|false|"
    Call SendMessage(getdlgitem(howner, lCtlId),%WM_USER + %our_add_property, 0,VarPtr(pi))

    pi.propName  =  "Visible"
    pi.curValue  =  "true"
    pi.nItemType =  %PIT_COMBO
    pi.cmbItems  =  "true|false|"
    Call SendMessage(getdlgitem(howner, lCtlId),%WM_USER + %our_add_property, 0,VarPtr(pi))

    pi.propName  =  "Fore. Color"
    pi.curValue  =  "RGB(255,255,0)"
    pi.nItemType =  %PIT_COLOR
    pi.cmbItems  =  ""
    Call SendMessage(getdlgitem(howner, lCtlId),%WM_USER + %our_add_property, 0,VarPtr(pi))

    pi.propName  =  "Back. Color"
    pi.curValue  =  "RGB(255,0,128)"
    pi.nItemType =  %PIT_COLOR
    pi.cmbItems  =  ""
    Call SendMessage(getdlgitem(howner, lCtlId),%WM_USER + %our_add_property, 0,VarPtr(pi))

    pi.propName  =  "Opaque"
    pi.curValue  =  "false"
    pi.nItemType =  %PIT_COMBO
    pi.cmbItems  =  "true|false|"
    Call SendMessage(getdlgitem(howner, lCtlId),%WM_USER + %our_add_property, 0,VarPtr(pi))

    pi.propName  =  "Auto. Scroll"
    pi.curValue  =  "true"
    pi.nItemType =  %PIT_COMBO
    pi.cmbItems  =  "true|false|"
    Call SendMessage(getdlgitem(howner, lCtlId),%WM_USER + %our_add_property, 0,VarPtr(pi))

    pi.propName  =  "Double Buffered"
    pi.curValue  =  "true"
    pi.nItemType =  %PIT_COMBO
    pi.cmbItems  =  "true|false|"
    Call SendMessage(getdlgitem(howner, lCtlId),%WM_USER + %our_add_property, 0,VarPtr(pi))

    pi.propName  =  "Font"
    pi.curValue  =  "MS San Sarif"
    pi.nItemType =  %PIT_FONT
    pi.cmbItems  =  ""
    Call SendMessage(getdlgitem(howner, lCtlId),%WM_USER + %our_add_property, 0,VarPtr(pi))

    pi.propName  = "Text"
    pi.curValue  =  "Big Bad Wolf!"
    pi.nItemType =  %PIT_EDIT
    pi.cmbItems  =  ""
    Call SendMessage(getdlgitem(howner, lCtlId),%WM_USER + %our_add_property, 0,VarPtr(pi))

    pi.propName  =  "Horiz. Align"
    pi.curValue  =  "CENTER"
    pi.nItemType =  %PIT_COMBO
    pi.cmbItems  =  "CENTER|LEFT|RIGHT|"
    Call SendMessage(getdlgitem(howner, lCtlId),%WM_USER + %our_add_property, 0,VarPtr(pi))

    pi.propName  =  "Vert. Align"
    pi.curValue  =  "CENTER"
    pi.nItemType =  %PIT_COMBO
    pi.cmbItems  =  "CENTER|TOP|BOTTOM|"
    Call SendMessage(getdlgitem(howner, lCtlId),%WM_USER + %our_add_property, 0,VarPtr(pi))

    pi.propName  =  "Bmp ,Icon"
    pi.curValue  =  "none"
    pi.nItemType =  %PIT_FILE
    pi.cmbItems  =  ""
    Call SendMessage(getdlgitem(howner, lCtlId),%WM_USER + %our_add_property, 0,VarPtr(pi))

    pi.propName  =  "Border Painted"
    pi.curValue  =  "false"
    pi.nItemType =  %PIT_COMBO
    pi.cmbItems  =  "true|false|"
    Call SendMessage(getdlgitem(howner, lCtlId),%WM_USER + %our_add_property, 0,VarPtr(pi))

    pi.propName  =  "Fill Content Area"
    pi.curValue  =  "true"
    pi.nItemType =  %PIT_COMBO
    pi.cmbItems  =  "true|false|"
    Call SendMessage(getdlgitem(howner, lCtlId),%WM_USER + %our_add_property, 0,VarPtr(pi))

    pi.propName  = "Focus Painted"
    pi.curValue  =  "true"
    pi.nItemType =  %PIT_COMBO
    pi.cmbItems  =  "true|false|"
    Call SendMessage(getdlgitem(howner, lCtlId),%WM_USER + %our_add_property, 0,VarPtr(pi))

    pi.propName  =  "Horiz. Text Pos."
    pi.curValue  =  "RIGHT"
    pi.nItemType =  %PIT_COMBO
    pi.cmbItems  =  "RIGHT|LEFT|CENTER|LEADING|TRAILING|"
    Call SendMessage(getdlgitem(howner, lCtlId),%WM_USER + %our_add_property, 0,VarPtr(pi))

End Sub


'--------------------------------------------------------------------------
' Load 16 Custom colors up for our ChooseColor() Dialog
'--------------------------------------------------------------------------
Sub InitializeCustomColors( pColors As Dword)

  Local lCounter As Long
  Local pl       As Long Ptr
  'load your custom colors...
  Randomize Timer
  pl = pColors
  For lCounter = 0 To 15
    'example colors,
    @pl = Rnd(0,16777215)
    Incr pl
  Next

End Sub
